module spence_func
contains
function r8_inits ( dos, nos, eta )

!*****************************************************************************80
!
!! R8_INITS initializes a Chebyshev series.
!
!  Licensing:
!
!    This code is distributed under the GNU LGPL license. 
!
!  Modified:
!
!    22 April 2016
!
!  Author:
!
!    FORTRAN90 version by John Burkardt.
!
!  Reference:
!
!    Roger Broucke,
!    Algorithm 446:
!    Ten Subroutines for the Manipulation of Chebyshev Series,
!    Communications of the ACM,
!    Volume 16, Number 4, April 1973, pages 254-256.
!
!  Parameters:
!
!    Input, real ( kind = 8 ) DOS(NOS), the Chebyshev coefficients.
!
!    Input, integer ( kind = 4 ) NOS, the number of coefficients.
!
!    Input, real ( kind = 8 ) ETA, the desired accuracy.
!
!    Output, integer ( kind = 4 ) R8_INITS, the number of terms of the 
!    series needed to ensure the requested accuracy.
!
  implicit none

  integer ( kind = 4 ) nos

  real ( kind = 8 ) dos(nos)
  real ( kind = 8 ) err 
  real ( kind = 8 ) eta
  integer ( kind = 4 ) i
  integer ( kind = 4 ) r8_inits
  integer ( kind = 4 ) value

  if ( nos < 1 ) then
    write ( *, '(a)' ) ' '
    write ( *, '(a)' ) 'R8_INITS - Fatal error!'
    write ( *, '(a)' ) '  Number of coefficients < 1.'
    stop 1
  end if

  if ( eta < dos(nos) ) then

    write ( *, '(a)' ) ' '
    write ( *, '(a)' ) 'R8_INITS - Warning!'
    write ( *, '(a)' ) '  ETA may be too small.'
    write ( *, '(a)' ) '  The requested accuracy cannot be guaranteed'
    write ( *, '(a)' ) '  even if all available coefficients are used.'
    value = nos

  else

    err = 0.0D+00

    do i = nos, 1, -1
      value = i
      err = err + abs ( dos(value) )
      if ( eta < err ) then
        exit
      end if
    end do

  end if

  r8_inits = value
end function r8_inits




double precision function r8_spence ( x )
!*****************************************************************************80
!
!! R8_SPENCE evaluates a form of Spence's function for an R8 argument.
!
!  Discussion:
!
!    This function evaluates a form of Spence's function defined by
!
!      f(x) = Integral ( 0 <= y <= x ) - log ( abs ( 1 - y ) ) / y dy
!
!  Licensing:
!
!    This code is distributed under the GNU LGPL license. 
!
!  Modified:
!
!    10 September 2011
!
!  Author:
!
!    Original FORTRAN77 version by Wayne Fullerton.
!    FORTRAN90 version by John Burkardt.
!
!  Reference:
!
!    Milton Abramowitz, Irene Stegun,
!    Handbook of Mathematical Functions, page 1004,
!    National Bureau of Standards, 1964,
!    ISBN: 0-486-61272-4,
!    LC: QA47.A34.
!
!    Wayne Fullerton,
!    Portable Special Function Routines,
!    in Portability of Numerical Software,
!    edited by Wayne Cowell,
!    Lecture Notes in Computer Science, Volume 57,
!    Springer 1977,
!    ISBN: 978-3-540-08446-4,
!    LC: QA297.W65.
!
!    K Mitchell,
!    Tables of the function Integral ( 0 < y < x ) - log | 1 - y | dy / y
!    with an account of some properties of this and related functions,
!    Philosophical Magazine,
!    Volume 40, pages 351-368, 1949.
!
!  Parameters:
!
!    Input, real ( kind = 8 ) X, the argument.
!
!    Output, real ( kind = 8 ) R8_SPENCE, Spence's function evaluated at X.
!
   implicit none
  real ( kind = 8 ) aln
  integer ( kind = 4 ) nspenc
  real ( kind = 8 ) pi26
!  real ( kind = 8 ) r8_csevl
!  integer ( kind = 4 ) r8_inits
!  real ( kind = 8 ) r8_mach
!  real ( kind = 8 ) r8_spence
  real ( kind = 8 ) spencs(38)
  real ( kind = 8 ),intent(in):: x
  real ( kind = 8 ) xbig

  save nspenc
  save pi26
  save spencs
  save xbig

  data spencs(  1) / +0.1527365598892405872946684910028D+00 /
  data spencs(  2) / +0.8169658058051014403501838185271D-01 /
  data spencs(  3) / +0.5814157140778730872977350641182D-02 /
  data spencs(  4) / +0.5371619814541527542247889005319D-03 /
  data spencs(  5) / +0.5724704675185826233210603054782D-04 /
  data spencs(  6) / +0.6674546121649336343607835438589D-05 /
  data spencs(  7) / +0.8276467339715676981584391689011D-06 /
  data spencs(  8) / +0.1073315673030678951270005873354D-06 /
  data spencs(  9) / +0.1440077294303239402334590331513D-07 /
  data spencs( 10) / +0.1984442029965906367898877139608D-08 /
  data spencs( 11) / +0.2794005822163638720201994821615D-09 /
  data spencs( 12) / +0.4003991310883311823072580445908D-10 /
  data spencs( 13) / +0.5823462892044638471368135835757D-11 /
  data spencs( 14) / +0.8576708692638689278097914771224D-12 /
  data spencs( 15) / +0.1276862586280193045989483033433D-12 /
  data spencs( 16) / +0.1918826209042517081162380416062D-13 /
  data spencs( 17) / +0.2907319206977138177795799719673D-14 /
  data spencs( 18) / +0.4437112685276780462557473641745D-15 /
  data spencs( 19) / +0.6815727787414599527867359135607D-16 /
  data spencs( 20) / +0.1053017386015574429547019416644D-16 /
  data spencs( 21) / +0.1635389806752377100051821734570D-17 /
  data spencs( 22) / +0.2551852874940463932310901642581D-18 /
  data spencs( 23) / +0.3999020621999360112770470379519D-19 /
  data spencs( 24) / +0.6291501645216811876514149171199D-20 /
  data spencs( 25) / +0.9933827435675677643803887752533D-21 /
  data spencs( 26) / +0.1573679570749964816721763805866D-21 /
  data spencs( 27) / +0.2500595316849476129369270954666D-22 /
  data spencs( 28) / +0.3984740918383811139210663253333D-23 /
  data spencs( 29) / +0.6366473210082843892691326293333D-24 /
  data spencs( 30) / +0.1019674287239678367077061973333D-24 /
  data spencs( 31) / +0.1636881058913518841111074133333D-25 /
  data spencs( 32) / +0.2633310439417650117345279999999D-26 /
  data spencs( 33) / +0.4244811560123976817224362666666D-27 /
  data spencs( 34) / +0.6855411983680052916824746666666D-28 /
  data spencs( 35) / +0.1109122433438056434018986666666D-28 /
  data spencs( 36) / +0.1797431304999891457365333333333D-29 /
  data spencs( 37) / +0.2917505845976095173290666666666D-30 /
  data spencs( 38) / +0.4742646808928671061333333333333D-31 /

  data nspenc / 0 /
  data pi26 / +1.644934066848226436472415166646025189219D+00 /
  data xbig / 0.0D+00 /

  if ( nspenc == 0 ) then
    nspenc = r8_inits ( spencs, 38, 0.1D+00 * r8_mach ( 3 ) )
    xbig = 1.0D+00 / r8_mach ( 3 )
  end if

  if ( x <= - xbig ) then

    aln = log ( 1.0D+00 - x )
    r8_spence = - pi26  - 0.5D+00 * aln * ( 2.0D+00 * log ( - x ) - aln )

  else if ( x <= - 1.0D+00 ) then

    aln = log ( 1.0D+00 - x )

    r8_spence = - pi26 - 0.5D+00 * aln * ( 2.0D+00 &
      * log ( - x ) - aln ) + ( 1.0D+00 + r8_csevl ( &
      4.0D+00 / ( 1.0D+00 - x ) - 1.0D+00, spencs, nspenc ) ) &
     / ( 1.0D+00 - x )

  else if ( x <= 0.0D+00 ) then

    r8_spence = - 0.5D+00 * log ( 1.0D+00 - x ) &
      * log ( 1.0D+00 - x ) - x * ( 1.0D+00 + r8_csevl ( &
      4.0D+00 * x / ( x - 1.0D+00 ) - 1.0D+00, spencs, nspenc ) ) &
      / ( x - 1.0D+00 )

  else if ( x <= 0.5D+00 ) then

    r8_spence = x * ( 1.0D+00 + r8_csevl ( 4.0D+00 * x - 1.0D+00, &
      spencs, nspenc ) )

  else if ( x < 1.0D+00 ) then

    r8_spence = pi26 - log ( x ) * log ( 1.0D+00 - x ) &
      - ( 1.0D+00 - x ) * ( 1.0D+00 + r8_csevl ( 4.0D+00 &
      * ( 1.0D+00 - x ) - 1.0D+00, spencs, nspenc ) )

  else if ( x == 1.0D+00 ) then

    r8_spence = pi26

  else if ( x <= 2.0D+00 ) then

    r8_spence = pi26 - 0.5D+00 * log ( x ) &
      * log ( ( x - 1.0D+00 ) * ( x - 1.0D+00 ) / x ) &
      + ( x - 1.0D+00 ) * ( 1.0D+00 + r8_csevl ( 4.0D+00 &
      * ( x - 1.0D+00 ) / x - 1.0D+00, spencs, nspenc ) ) / x

  else if ( x < xbig ) then

    r8_spence = 2.0D+00 * pi26 - 0.5D+00 * log ( x ) * log ( x ) &
      - ( 1.0D+00 + r8_csevl ( 4.0D+00 / x - 1.0D+00, spencs, &
      nspenc ) ) / x

  else

    r8_spence = 2.0D+00 * pi26 - 0.5D+00 * log ( x ) * log ( x )

  end if

end function r8_spence

function r8_csevl ( x, a, n )

!*****************************************************************************80
!
!! R8_CSEVL evaluates a Chebyshev series.
!
!  Licensing:
!
!    This code is distributed under the GNU LGPL license. 
!
!  Modified:
!
!    10 September 2011
!
!  Author:
!
!    FORTRAN90 version by John Burkardt.
!
!  Reference:
!
!    Roger Broucke,
!    Algorithm 446:
!    Ten Subroutines for the Manipulation of Chebyshev Series,
!    Communications of the ACM,
!    Volume 16, Number 4, April 1973, pages 254-256.
!
!  Parameters:
!
!    Input, real ( kind = 8 ) X, the evaluation point.
!
!    Input, real ( kind = 8 ) A(N), the Chebyshev coefficients.
!
!    Input, integer ( kind = 4 ) N, the number of Chebyshev coefficients.
!
!    Output, real ( kind = 8 ) R8_CSEVL, the Chebyshev series evaluated at X.
!
  implicit none

  integer ( kind = 4 ) n

  real ( kind = 8 ) a(n)
  real ( kind = 8 ) b0
  real ( kind = 8 ) b1
  real ( kind = 8 ) b2
  integer ( kind = 4 ) i
  real ( kind = 8 ) r8_csevl
  real ( kind = 8 ) twox
  real ( kind = 8 ) x

  if ( n < 1 ) then
    write ( *, '(a)' ) ' '
    write ( *, '(a)' ) 'R8_CSEVL - Fatal error!'
    write ( *, '(a)' ) '  Number of terms <= 0.'
    stop 1
  end if

  if ( 1000 < n ) then
    write ( *, '(a)' ) ' '
    write ( *, '(a)' ) 'R8_CSEVL - Fatal error!'
    write ( *, '(a)' ) '  1000 < Number of terms.'
    stop 1
  end if

  if ( x < -1.1D+00 .or. 1.1D+00 < x ) then
    write ( *, '(a)' ) ' '
    write ( *, '(a)' ) 'R8_CSEVL - Fatal error!'
    write ( *, '(a)' ) '  X outside [-1,+1]'
    write ( *, '(a,g14.6)' ) '  X = ', x
    stop 1
  end if

  twox = 2.0D+00 * x
  b1 = 0.0D+00
  b0 = 0.0D+00

  do i = n, 1, -1
    b2 = b1
    b1 = b0
    b0 = twox * b1 - b2 + a(i)
  end do

  r8_csevl = 0.5D+00 * ( b0 - b2 )

end function r8_csevl

function r8_mach ( i )

!*****************************************************************************80
!
!! R8_MACH returns real ( kind = 8 ) real machine-dependent constants.
!
!  Discussion:
!
!    R8_MACH can be used to obtain machine-dependent parameters
!    for the local machine environment.  It is a function
!    with one input argument, and can be called as follows:
!
!      D = R8_MACH ( I )
!
!    where I=1,...,5.  The output value of D above is
!    determined by the input value of I:.
!
!    R8_MACH ( 1) = B^(EMIN-1), the smallest positive magnitude.
!    R8_MACH ( 2) = B^EMAX*(1 - B^(-T)), the largest magnitude.
!    R8_MACH ( 3) = B^(-T), the smallest relative spacing.
!    R8_MACH ( 4) = B^(1-T), the largest relative spacing.
!    R8_MACH ( 5) = LOG10(B)
!
!  Licensing:
!
!    This code is distributed under the GNU LGPL license. 
!
!  Modified:
!
!    25 April 2007
!
!  Author:
!
!    Original FORTRAN77 version by Phyllis Fox, Andrew Hall, Norman Schryer.
!    FORTRAN90 version by John Burkardt.
!
!  Reference:
!
!    Phyllis Fox, Andrew Hall, Norman Schryer,
!    Algorithm 528:
!    Framework for a Portable Library,
!    ACM Transactions on Mathematical Software,
!    Volume 4, Number 2, June 1978, page 176-188.
!
!  Parameters:
!
!    Input, integer ( kind = 4 ) I, the index of the desired constant.
!
!    Output, real ( kind = 8 ) R8_MACH, the value of the constant.
!
  implicit none

  real ( kind = 8 ) r8_mach
  integer ( kind = 4 ) i

  if ( i < 1 ) then
    write ( *, '(a)' ) ' '
    write ( *, '(a)' ) 'R8_MACH - Fatal error!'
    write ( *, '(a)' ) '  The input argument I is out of bounds.'
    write ( *, '(a)' ) '  Legal values satisfy 1 <= I <= 5.'
    write ( *, '(a,i12)' ) '  I = ', i
    r8_mach = 0.0D+00
    stop 1
  else if ( i == 1 ) then
    r8_mach = 4.450147717014403D-308
  else if ( i == 2 ) then
    r8_mach = 8.988465674311579D+307
  else if ( i == 3 ) then
    r8_mach = 1.110223024625157D-016
  else if ( i == 4 ) then
    r8_mach = 2.220446049250313D-016
  else if ( i == 5 ) then
    r8_mach = 0.301029995663981D+000
  else if ( 5 < i ) then
    write ( *, '(a)' ) ' '
    write ( *, '(a)' ) 'R8_MACH - Fatal error!'
    write ( *, '(a)' ) '  The input argument I is out of bounds.'
    write ( *, '(a)' ) '  Legal values satisfy 1 <= I <= 5.'
    write ( *, '(a,i12)' ) '  I = ', i
    r8_mach = 0.0D+00
    stop 1
  end if
end function r8_mach



end module spence_func

